@@ -1,9 +1,10 @@
 //! Functions relating to ECU Identification
 
-use std::fmt::format;
-
-use crate::{DiagError, DiagServerResult, helpers::{bcd_decode, bcd_decode_slice}, kwp2000::{KWP2000Command, Kwp2000DiagnosticServer}};
-
+use crate::{
+    helpers::{bcd_decode, bcd_decode_slice},
+    kwp2000::{KWP2000Command, Kwp2000DiagnosticServer},
+    DiagError, DiagServerResult, DiagnosticServer
+};
 
 #[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
 
@@ -11,12 +12,11 @@ use crate::{DiagError, DiagServerResult, helpers::{bcd_decode, bcd_decode_slice}
 pub struct DiagnosticInfo([u8; 2]);
 
 impl DiagnosticInfo {
-
     /// Returns true if the ECU software is production. If this
     /// function returns false, then the software running on the ECU
     /// is considered to be development only
     pub fn is_production_ecu(&self) -> bool {
-        self.0[0] & 0b10000000 == 0 
+        self.0[0] & 0b10000000 == 0
     }
 
     /// Returns the unique ECU ID allocated by DCX for the specific ECU
@@ -33,7 +33,7 @@ impl DiagnosticInfo {
 
     /// Returns the entire diagnostic info ID as a u16
     pub fn get_info_id(&self) -> u16 {
-        return (self.0[0] as u16) << 8 | self.0[1] as u16
+        return (self.0[0] as u16) << 8 | self.0[1] as u16;
     }
 }
 
@@ -63,7 +63,7 @@ pub struct DcsEcuIdent {
     /// Month of the year the ECU was manufactured
     pub ecu_production_month: u8,
     /// Day of the month the ECU was manufactured
-    pub ecu_production_day: u8   
+    pub ecu_production_day: u8,
 }
 
 /// Identification structure read with [read_dcx_mmc_identification]
@@ -80,21 +80,20 @@ pub struct DcxMmcEcuIdent {
     /// Software version. Formatted as XX.YY.ZZ
     pub sw_version: String,
     /// ECU Part number (10 character alpha-numeric string)
-    pub part_number: String
+    pub part_number: String,
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
 /// ECU Code block fingerprint
 pub struct ModuleInformation {
     /// Number logical blocks marked to be erased.
-    /// 
+    ///
     /// ## Special values
     /// * 0x00 - Perform no erase
     /// * 0xFE - Erase all blocks
     pub active_logical_blocks: u8,
     /// Information on each block
     pub module_info: Vec<ModuleBlockInformation>,
-
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
@@ -109,7 +108,7 @@ pub struct ModuleBlockInformation {
     /// Programmed Day
     pub programming_date_day: u8,
     /// Tester serial number who programmed the block
-    pub tester_serial_number: String
+    pub tester_serial_number: String,
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
@@ -118,7 +117,7 @@ pub struct SoftwareBlockIdentification {
     /// ECU origin
     pub origin: u8,
     /// Identification of each block of the software
-    pub blocks: Vec<BlockIdentification>
+    pub blocks: Vec<BlockIdentification>,
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
@@ -131,14 +130,17 @@ pub struct BlockIdentification {
     /// Code block Software version. Formatted as XX.YY.ZZ
     pub sw_version: String,
     /// Code block part number
-    pub part_number: String
+    pub part_number: String,
 }
 
 /// Reads DCS ECU identification from ECU
-pub fn read_dcs_identification(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<DcsEcuIdent> {
-    let res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x86])?;
+pub fn read_dcs_identification(
+    server: &mut Kwp2000DiagnosticServer,
+) -> DiagServerResult<DcsEcuIdent> {
+    let res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x86])?;
     if res.len() != 18 {
-        return Err(DiagError::InvalidResponseLength)
+        return Err(DiagError::InvalidResponseLength);
     }
     Ok(DcsEcuIdent {
         part_number: bcd_decode_slice(&res[2..7], None),
@@ -154,12 +156,14 @@ pub fn read_dcs_identification(server: &mut Kwp2000DiagnosticServer) -> DiagServ
     })
 }
 
-
 /// Reads DCX/MMC ECU identification from ECU
-pub fn read_dcx_mmc_identification(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<DcxMmcEcuIdent> {
-    let res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x87])?;
+pub fn read_dcx_mmc_identification(
+    server: &mut Kwp2000DiagnosticServer,
+) -> DiagServerResult<DcxMmcEcuIdent> {
+    let res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x87])?;
     if res.len() != 22 {
-        return Err(DiagError::InvalidResponseLength)
+        return Err(DiagError::InvalidResponseLength);
     }
     Ok(DcxMmcEcuIdent {
         ecu_origin: res[2],
@@ -173,57 +177,67 @@ pub fn read_dcx_mmc_identification(server: &mut Kwp2000DiagnosticServer) -> Diag
 
 /// Reads the original VIN programmed onto the ECU from the manufacturer
 pub fn read_original_vin(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<String> {
-    let res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x88])?;
+    let res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x88])?;
     Ok(String::from_utf8_lossy(&res[2..]).to_string())
 }
 
 /// Reads the unique diagnostic variant code of the ECU
 pub fn read_diagnostic_variant_code(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<u32> {
-    let res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x89])?;
+    let res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x89])?;
     if res.len() != 6 {
-        return Err(DiagError::InvalidResponseLength)
+        return Err(DiagError::InvalidResponseLength);
     }
     Ok((res[2] as u32) << 24 | (res[3] as u32) << 16 | (res[4] as u32) << 8 | res[5] as u32)
 }
 
 /// Reads the current VIN stored on the ECU
 pub fn read_current_vin(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<String> {
-    let res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x90])?;
+    let res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x90])?;
     Ok(String::from_utf8_lossy(&res[2..]).to_string())
 }
 
 /// Reads the OBD Calibration ID from the ECU.
 pub fn read_calibration_id(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<String> {
-    let res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x96])?;
+    let res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x96])?;
     Ok(String::from_utf8_lossy(&res[2..]).to_string())
 }
 
 /// Reads the calibration verification number from the ECU
 pub fn read_cvn(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<[u8; 4]> {
-    let res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x97])?;
+    let res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x97])?;
     if res.len() != 6 {
-        return Err(DiagError::InvalidResponseLength)
+        return Err(DiagError::InvalidResponseLength);
     }
     Ok([res[2], res[3], res[4], res[5]])
 }
 
-
 fn decode_module_info(res: &mut Vec<u8>) -> DiagServerResult<ModuleInformation> {
     let active_logical_blocks = res[3];
     res.drain(0..4);
     let mut list_of_blocks: Vec<ModuleBlockInformation> = Vec::new();
 
     if res.len() % 8 != 0 {
-        return Err(DiagError::InvalidResponseLength)
+        return Err(DiagError::InvalidResponseLength);
     }
 
     for i in (0..res.len()).step_by(8) {
         list_of_blocks.push(ModuleBlockInformation {
             tool_supplier_id: res[i],
-            programming_date_year: u8::from_str_radix(&bcd_decode(res[i+1]), 10).unwrap(),
-            programming_date_month: u8::from_str_radix(&bcd_decode(res[i+2]), 10).unwrap(),
-            programming_date_day: u8::from_str_radix(&bcd_decode(res[i+3]), 10).unwrap(),
-            tester_serial_number: format!("{:02X}{:02X}{:02X}{:02X}", res[i+4],res[i+5],res[i+6],res[i+7]),
+            programming_date_year: u8::from_str_radix(&bcd_decode(res[i + 1]), 10).unwrap(),
+            programming_date_month: u8::from_str_radix(&bcd_decode(res[i + 2]), 10).unwrap(),
+            programming_date_day: u8::from_str_radix(&bcd_decode(res[i + 3]), 10).unwrap(),
+            tester_serial_number: format!(
+                "{:02X}{:02X}{:02X}{:02X}",
+                res[i + 4],
+                res[i + 5],
+                res[i + 6],
+                res[i + 7]
+            ),
         })
     }
 
@@ -234,14 +248,20 @@ fn decode_module_info(res: &mut Vec<u8>) -> DiagServerResult<ModuleInformation>
 }
 
 /// Reads module information from the ECU's code block
-pub fn read_ecu_code_fingerprint(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<ModuleInformation> {
-    let mut res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9A])?;
+pub fn read_ecu_code_fingerprint(
+    server: &mut Kwp2000DiagnosticServer,
+) -> DiagServerResult<ModuleInformation> {
+    let mut res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9A])?;
     decode_module_info(&mut res)
 }
 
 /// Reads module information from the ECU's data block
-pub fn read_ecu_data_fingerprint(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<ModuleInformation> {
-    let mut res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9B])?;
+pub fn read_ecu_data_fingerprint(
+    server: &mut Kwp2000DiagnosticServer,
+) -> DiagServerResult<ModuleInformation> {
+    let mut res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9B])?;
     decode_module_info(&mut res)
 }
 
@@ -251,45 +271,54 @@ fn decode_block_ident(res: &mut Vec<u8>) -> DiagServerResult<SoftwareBlockIdenti
     res.drain(0..4);
 
     if res.len() % 17 != 0 {
-        return Err(DiagError::InvalidResponseLength)
+        return Err(DiagError::InvalidResponseLength);
     }
 
-    let mut blocks : Vec<BlockIdentification> = Vec::new();
+    let mut blocks: Vec<BlockIdentification> = Vec::new();
     for x in (0..res.len()).step_by(17) {
         blocks.push(BlockIdentification {
             supplier_id: res[x],
-            diag_info: DiagnosticInfo([res[x+1], res[x+2]]),
-            sw_version: bcd_decode_slice(&res[x+4..x+7], Some(".")),
-            part_number: bcd_decode_slice(&res[x+8..x+17], None),
+            diag_info: DiagnosticInfo([res[x + 1], res[x + 2]]),
+            sw_version: bcd_decode_slice(&res[x + 4..x + 7], Some(".")),
+            part_number: bcd_decode_slice(&res[x + 8..x + 17], None),
         })
     }
 
-    Ok(SoftwareBlockIdentification {
-        origin,
-        blocks,
-    })
+    Ok(SoftwareBlockIdentification { origin, blocks })
 }
 
 /// Reads code identification information from the ECU's code block
-pub fn read_ecu_code_software_id(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<SoftwareBlockIdentification> {
-    let mut res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9C])?;
+pub fn read_ecu_code_software_id(
+    server: &mut Kwp2000DiagnosticServer,
+) -> DiagServerResult<SoftwareBlockIdentification> {
+    let mut res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9C])?;
     decode_block_ident(&mut res)
 }
 
 /// Reads code identification information from the ECU's data block
-pub fn read_ecu_data_software_id(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<SoftwareBlockIdentification> {
-    let mut res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9D])?;
+pub fn read_ecu_data_software_id(
+    server: &mut Kwp2000DiagnosticServer,
+) -> DiagServerResult<SoftwareBlockIdentification> {
+    let mut res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9D])?;
     decode_block_ident(&mut res)
 }
 
 /// Reads code identification information from the ECU's boot block
-pub fn read_ecu_boot_software_id(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<SoftwareBlockIdentification> {
-    let mut res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9E])?;
+pub fn read_ecu_boot_software_id(
+    server: &mut Kwp2000DiagnosticServer,
+) -> DiagServerResult<SoftwareBlockIdentification> {
+    let mut res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9E])?;
     decode_block_ident(&mut res)
 }
- 
+
 /// Reads code identification information from the ECU's boot block
-pub fn read_ecu_boot_fingerprint(server: &mut Kwp2000DiagnosticServer) -> DiagServerResult<ModuleInformation> {
-    let mut res = server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9F])?;
+pub fn read_ecu_boot_fingerprint(
+    server: &mut Kwp2000DiagnosticServer,
+) -> DiagServerResult<ModuleInformation> {
+    let mut res =
+        server.execute_command_with_response(KWP2000Command::ReadECUIdentification, &[0x9F])?;
     decode_module_info(&mut res)
-}
\ No newline at end of file
+}
