@@ -1,4 +1,5 @@
 #![deny(missing_docs, missing_debug_implementations)]
+#![allow(dead_code)]
 
 //! A crate which provides the most common ECU diagnostic protocols used by modern ECUs in vehicles.
 //!
@@ -38,9 +39,11 @@
 //!
 
 use channel::ChannelError;
+use hardware::HardwareError;
 
 pub mod channel;
 pub mod dtc;
+pub mod hardware;
 pub mod kwp2000;
 pub mod obd2;
 pub mod uds;
@@ -65,20 +68,27 @@ pub enum DiagError {
     ServerNotRunning,
     /// ECU Responded with a message, but the length was incorrect
     InvalidResponseLength,
+    /// A parameter given to the function is invalid. Check the function's documentation
+    /// for more information
+    ParameterInvalid,
     /// Error with underlying communication channel
     ChannelError(ChannelError),
     /// Denotes a TODO action (Non-implemented function stub)
     /// This will be removed in Version 1
     NotImplemented(String),
+    /// Device hardware error
+    HardwareError(HardwareError),
 }
+
 impl std::fmt::Display for DiagError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        match self {
+        match &self {
             DiagError::NotSupported => write!(f, "request not supported"),
             DiagError::ECUError(err) => write!(f, "ECU error 0x{:02X}", err),
             DiagError::EmptyResponse => write!(f, "ECU provided an empty response"),
             DiagError::WrongMessage => write!(f, "ECU response message did not match request"),
             DiagError::ServerNotRunning => write!(f, "diagnostic server not running"),
+            DiagError::ParameterInvalid => write!(f, "a parameter provided was invalid"),
             DiagError::InvalidResponseLength => {
                 write!(f, "ECU response message was of invalid length")
             }
@@ -86,15 +96,17 @@ impl std::fmt::Display for DiagError {
             DiagError::NotImplemented(s) => {
                 write!(f, "server encountered an unimplemented function: {}", s)
             }
+            &DiagError::HardwareError(e) => write!(f, "Hardware error: {}", e),
         }
     }
 }
+
 impl std::error::Error for DiagError {
     fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
-        if let DiagError::ChannelError(err) = self {
-            Some(err)
-        } else {
-            None
+        match &self {
+            DiagError::ChannelError(e) => Some(e),
+            DiagError::HardwareError(e) => Some(e),
+            _ => None,
         }
     }
 }
@@ -105,6 +117,12 @@ impl From<ChannelError> for DiagError {
     }
 }
 
+impl From<HardwareError> for DiagError {
+    fn from(x: HardwareError) -> Self {
+        Self::HardwareError(x)
+    }
+}
+
 #[derive(Debug)]
 /// Diagnostic server event
 pub enum ServerEvent<'a, SessionState, RequestType> {
@@ -146,6 +164,30 @@ pub trait ServerEventHandler<SessionState, RequestType>: Send + Sync {
     fn on_event(&mut self, e: ServerEvent<SessionState, RequestType>);
 }
 
+/// Base trait for diagnostic servers
+pub trait DiagnosticServer<CommandType> {
+
+    /// Sends a command to the ECU, and doesn't poll for its response
+    fn execute_command(&mut self, cmd: CommandType, args: &[u8]) -> DiagServerResult<()>;
+    /// Sends a command to the ECU, and polls for its response
+    fn execute_command_with_response(&mut self, cmd: CommandType, args: &[u8]) -> DiagServerResult<Vec<u8>>;
+    /// Sends an arbitrary byte array to the ECU, and doesn't poll for its response
+    fn send_byte_array(&mut self, bytes: &[u8]) -> DiagServerResult<()>;
+    /// Sends an arbitrary byte array to the ECU, and polls for its response
+    fn send_byte_array_with_response(&mut self, bytes: &[u8]) -> DiagServerResult<Vec<u8>>;
+
+    /// Returns if the diagnostic server is running or not
+    fn is_server_running(&self) -> bool;
+
+    /// Sets the maximum number of retries to send a command to the ECU
+    /// if a failure occurs during transmission of the command to the ECU
+    fn set_repeat_count(&mut self, count: u32);
+
+    /// Sets the minimum interval in milliseconds
+    /// between a command failure and an attempted repeat transmission
+    fn set_repeat_interval_count(&mut self, interval_ms: u32);
+}
+
 /// Basic diagnostic server settings
 pub trait BaseServerSettings {
     /// Gets the write timeout for sending messages to the servers channel
